local Grid2AlertOptions = {}
Grid2Options.plugins["Grid2AlertOptions"] = Grid2AlertOptions

local L = LibStub("AceLocale-3.0"):GetLocale("Grid2Options")
local LG = LibStub("AceLocale-3.0"):GetLocale("Grid2")
local DBL = LibStub:GetLibrary("LibDBLayers-1.0")


-- dblData - data table for the mod
-- versions - table of version info
-- layers to create defaults for
function Grid2AlertOptions:MakeDefaults(dblData, versions, layers)
	for i, layer in pairs(layers) do
		if (layer == "account") then
			if (versions.Grid2AlertOptions < 1) then
				DBL:SetupLayerObject(dblData, "alerts", layer, "aggro", {
						status = "threat",
						type = "self-gain-aggro",
						message = "Aggro !",
						sound = "threat",
						min_pause = 0.5,
						name = "aggro",
				})

				versions.Grid2AlertOptions = 1
			end
		elseif (layer == "deathknight") then
			if (versions.deathknight.Grid2AlertOptions < 1) then

				versions.deathknight.Grid2AlertOptions = 1
			end
		elseif (layer == "druid") then
			if (versions.druid.Grid2AlertOptions < 1) then
				DBL:SetupLayerObject(dblData, "alerts", layer, "detox", {
						status = {"debuff-Curse", "debuff-Poison"},
						type = "any-gain",
						message = nil,
						sound = "Detox",
						min_pause = 0.5,
						name = "detox",
				})

				versions.druid.Grid2AlertOptions = 1
			end
		elseif (layer == "hunter") then
			if (versions.hunter.Grid2AlertOptions < 1) then

				versions.hunter.Grid2AlertOptions = 1
			end
		elseif (layer == "mage") then
			if (versions.mage.Grid2AlertOptions < 1) then
				DBL:SetupLayerObject(dblData, "alerts", layer, "detox", {
						status = {"debuff-Curse"},
						type = "any-gain",
						message = nil,
						sound = "Detox",
						min_pause = 0.5,
						name = "detox",
				})

				versions.mage.Grid2AlertOptions = 1
			end
		elseif (layer == "paladin") then
			if (versions.paladin.Grid2AlertOptions < 1) then
				DBL:SetupLayerObject(dblData, "alerts", layer, "detox", {
						status = {"debuff-Magic", "debuff-Poison", "debuff-Disease"},
						type = "any-gain",
						message = nil,
						sound = "Detox",
						min_pause = 0.5,
						name = "detox",
				})

				versions.paladin.Grid2AlertOptions = 1
			end
		elseif (layer == "priest") then
			if (versions.priest.Grid2AlertOptions < 1) then
				DBL:SetupLayerObject(dblData, "alerts", layer, "detox", {
						status = {"debuff-Magic", "debuff-Disease"},
						type = "any-gain",
						message = nil,
						sound = "Detox",
						min_pause = 0.5,
						name = "detox",
				})

				versions.priest.Grid2AlertOptions = 1
			end
		elseif (layer == "rogue") then
			if (versions.rogue.Grid2AlertOptions < 1) then

				versions.rogue.Grid2AlertOptions = 1
			end
		elseif (layer == "shaman") then
			if (versions.shaman.Grid2AlertOptions < 1) then
				local statusList = {"debuff-Disease", "debuff-Poison"}
				if select(5, GetTalentInfo(3, 18)) > 0 then
					table.insert(statusList, "debuff-Curse")
				end
				DBL:SetupLayerObject(dblData, "alerts", layer, "detox", {
						status = statusList,
						type = "any-gain",
						message = nil,
						sound = "Detox",
						min_pause = 0.5,
						name = "detox",
				})

				versions.shaman.Grid2AlertOptions = 1
			end
		elseif (layer == "warlock") then
			if (versions.warlock.Grid2AlertOptions < 1) then

				versions.warlock.Grid2AlertOptions = 1
			end
		elseif (layer == "warrior") then
			if (versions.warrior.Grid2AlertOptions < 1) then

				versions.warrior.Grid2AlertOptions = 1
			end
		end
	end
end

function Grid2AlertOptions.UpgradeDefaults(dblData)
	local versionsSrc = dblData.versionsSrc
	local layerOrder = Grid2Options.layerOrder
	local layers = Grid2Options.layers
	
	DBL:InitializeObjectType(dblData, "alerts", layerOrder)
print("Grid2AlertOptions UpgradeDefaults")

	Grid2AlertOptions:MakeDefaults(dblData, versionsSrc, layers)
end



local function MakeGroupOptions(reset)
	local options = {
		resetAlerts = {
			type = "execute",
			order = 11,
			name = L["Reset"],
			desc = L["Reset to defaults."],
			func = ResetAlerts,
		},
	}
	Grid2Options:AddElementGroup("alerts", options, 90, reset)
end

local function MakeAlertOptions(alert)
	local baseKey = alert.name
	local options = {
		-- sound = {
			-- type = "input",
			-- order = 40,
			-- name = L["Sound"],
			-- desc = L["Sound to play when the alert is triggered."],
			-- usage = L["<sound>"],
			-- get = function ()
				-- return module.db.profile.sound or ""
			-- end,
			-- set = function (v)
				-- module.db.profile.sound = (v ~= "" and v)
			-- end,
			-- validate = media:List("sound"),
		-- },
		message = {
			type = "input",
			order = 50,
			name = L["Message to display"],
			desc = L["Message to display, set to off to disable."],
			usage = "<string>",
			get = function ()
				return alert.message or L["off"]
			end,
			set = function (v)
				local value = v ~= L["off"] and v
				alert.min_pause = value
				DBL:GetOptionsDbx(Grid2.dblData, "alerts", baseKey).min_pause = value
			end,
		},
		minPause = {
			type = "range",
			order = 60,
			name = L["Minimum pause"],
			desc = L["Minimum pause between two of this kind of alert, in second."],
			min = 0,
			max = 2,
			step = 0.1,
			get = function ()
				return alert.min_pause
			end,
			set = function (v)
				alert.min_pause = v
				DBL:GetOptionsDbx(Grid2.dblData, "alerts", baseKey).min_pause = v
			end,
		},
	}

	if Grid2Options.AddMediaOption then
		local soundOption = {
			type = "select",
			order = 70,
			name = L["Sound"],
			desc = L["Sound to play when the alert is triggered."],
		}
		Grid2Options:AddMediaOption("sound", soundOption)
		local values = soundOption.values
		soundOption.get = function ()
			local index
			for i, handle in ipairs(values) do
				if (alert.font == handle) then
					index = i
					break
				end
			end
			return index
		end
		soundOption.set = function (_, v)
			local handle = values[v]
			alert.sound = handle
			DBL:GetOptionsDbx(Grid2.dblData, "alerts", baseKey).sound = handle
			local sound = media:Fetch("sound", handle)
		end

		options.sound = soundOption
	end

	Grid2Options:AddElement("alerts", alert, options)
end

local prev_MakeOptions = Grid2Options.MakeOptions
function Grid2Options:MakeOptions(dblData, reset, ...)
	MakeGroupOptions(reset)

	self:AddOptionHandler("any-gain", MakeAlertOptions)
	self:AddOptionHandler("self-gain-aggro", MakeAlertOptions)

	local objects = DBL:GetOptionsObjects(dblData, "alerts")
	local setup = DBL:GetRuntimeSetup(dblData, "alerts")

	for baseKey, layer in pairs(setup) do
		local alert = Grid2.alerts[baseKey]
		local dbx = objects[layer][baseKey]
		if (dbx) then
			local funcMakeOptions = Grid2Options.typeMakeOptions[dbx.type]
			if (funcMakeOptions) then
				funcMakeOptions(alert)
			else
				print("    **MakeAlertOptions no funcMakeOptions for", layer, baseKey, dbx.type)
			end
		else
			print("    **MakeAlertOptions no dbx for", layer, baseKey)
		end
	end

	prev_MakeOptions(self, dblData, reset, ...)
end

local prev_FlattenDefaults = Grid2Options.FlattenDefaults
function Grid2Options:FlattenDefaults(dblData, ...)
	DBL:FlattenSetupType(dblData, "alerts")
--	DBL:FlattenMap(dblData, "alertMap", "alerts", "statuses")

	prev_FlattenDefaults(self, dblData, ...)
end
